#include "graph.h"

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Project: perform projection on each point, must be done on copies of
//	actual graph object.

void graph::Project(void){
   int x,y,z;
    for(int i=0;i<numpts;i++){
	x=pts[i]->operator[](0);
	y=pts[i]->operator[](1);
	z=pts[i]->operator[](2);
	x=(x*VIEW_DISTANCE)/(z-VIEW_DISTANCE);
	y=(y*VIEW_DISTANCE)/(z-VIEW_DISTANCE);
	pts[i]->operator[](0)=x;
	pts[i]->operator[](1)=y;
    } // end for
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// class destructor: de-allocates memory for edge, vertex arrays, and their
//		contents.
	graph::~graph(){
		// delete all points:
		if(initialized==TRUE){
		   for(int i=0;i<numpts;i++){
			delete pts[i];
		      };
		  };
	       // delete point array:
		   delete pts;
	       // delete all edges:
		   for(int i=0;i<numedges;i++){
			delete edges[i];
		      };
		   delete edges;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Draw : draws all edges in edge array.

void	graph::Draw(void){
  int lx,ly,rx,ry;
  int curcolor;
if(colored==TRUE){
   curcolor=edges[0]->color;
   ::setcolor(curcolor);
 for(int i=0;i<numedges;i++){
   lx=pts[edges[i]->from]->operator[](0)+Tx;
   ly=pts[edges[i]->from]->operator[](1)+Ty;
   rx=pts[edges[i]->to]->operator[](0)+Tx;
   ry=pts[edges[i]->to]->operator[](1)+Ty;
   if(edges[i]->color != curcolor){
    ::setcolor(edges[i]->color);
    curcolor=edges[i]->color;
   }
   ::line(lx,ly,rx,ry);
  } // end for
  } else {
   for(int i=0;i<numedges;i++){
   lx=pts[edges[i]->from]->operator[](0)+Tx;
   ly=pts[edges[i]->from]->operator[](1)+Ty;
   rx=pts[edges[i]->to]->operator[](0)+Tx;
   ry=pts[edges[i]->to]->operator[](1)+Ty;
   ::line(lx,ly,rx,ry);
   } // end for
  } // end else
}; // end draw()

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// apply matrix to each point in array.

void graph::Apply(matrix& m){
	for(int i=0;i<numpts;i++){
		*pts[i] = *pts[i] * m;
	   };
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Replace_edge_component:
//	in the case of a deletion or insertion, old edge values may be rendered
//	inaccurate (as indices into the vertex array).
// 	Replace_edge_component finds all old values in the edge array and
//	replaces them with their new values.
//	the ellipses is used so as all values to be replaced can be replaced
//	at once, with the parameters expected as pairs of integers, the
//	first in each pair representing the old value, and the second
//	is the value to replace that old value.
//      first argument 'numargs', is the total number of arguments.
//	returns 1 if replacement was made, else return 0

int 	graph::Replace_edge_component(int numargs,int old,int replace,...){
	int retval=0;
// set up array to hold all values to look for and values to replace with.
   int **search = (int**) new int[numargs-1];
// set up variable size parameter list macros:
 va_list ap;
// point to first parameter
va_start(ap, numargs);
// read in all subsequent parameters into search array.
for(int i=0;i<numargs-1;i++){
	*search[i] =va_arg(ap,int);
 }
va_end(ap); // clean up pointers after all parameters have been read in.

// search through edge list to find values to replace and replace them.
for(i=0;i<numedges;i++){
    for(int j=0;j<numargs-1;j+=2){
      // look for value in both fields:
	if(edges[i]->to==*search[j]){   // find value
	     edges[i]->to=*search[j+1]; // replace it
	     retval=1;
	  } else
	if(edges[i]->from==*search[j]){
	     edges[i]->from=*search[j+1];
	     retval=1;
	  }  // end else
     } // end searches list
} // end edges list.
return retval;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Insert_pt:   inserts point into point array.
//      assumes memory has been allocated already for new point object.

void	graph::Insert_pt(vector* ptr,int index){
	assert((index >=0)&&(index<max_pts));
       // if there is not enough room in array:
	if(max_pts < numpts){
		enlarge_vertex_array_by(1);
	  };
	// redirect all pointers above insertion point.
//	for(int i=numpts;i>index;i--){
//	      pts[i]=pts[i-1];
//	   };
	pts[index]=ptr;
	numpts++;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Delete_pt:	deletes point indexed by 'index'.

void    graph::Delete_pt(int index){
	assert((index >=0)&& (index <numpts));
	delete pts[index];
  // reassign all ponters above current point index to maintain array struct.
	for(int i=index;i<numpts;i++){
		pts[i]=pts[i+1];
	    };
	numpts--;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Replace_pt:	replaces point indexed by 'index' with point pointed
//	to by 'ptr'. Returns pointer to point being replaced.
//  ASSUMES memory has been allocated for the new point.

vector* graph::Replace_pt(int index,vector *ptr){
	  assert((index >=0)&&(index < numpts));
	  vector* retval=pts[index];
	  pts[index]=ptr;
	  return retval;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Add_pt : adds a point to the end of the array ( so as not to disrupt the
//		edge list).
//	ASSUMES memory has been allocate for pt. being added.

void    graph::Add_pt(vector *ptr){
      // if there is not enough room for new point, enlarge the array.
	 if(numpts >= max_pts){
		enlarge_vertex_array_by(1);
	   };
      // point to new point
	 pts[numpts]=ptr;
	 numpts++;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//Insert_edge: inserts indexed edge with new edge.
//	ASSUMES memory has been allocated for new edge

void    graph::Insert_edge(edge *ptr,int index){
	assert((index >= 0) && (index < max_edges));

     //	edge* retval = edges[index];
	edges[index]=ptr;
        numedges++;
     //	return retval;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Delete_edge: deletes edge indexed by 'index' in edge array.
//	relocates all edges indexed numerically above the edge
//	to be deleted to preserve edge array structure.

void    graph::Delete_edge(int index){
	assert((index >=0) && (index < numedges));
	delete edges[index];
      // bring each pointer in edge array to point to the one above it.
	for(int i=index;i<numedges-1;i++){
	   edges[i] = edges[i+1];
	 }
      // indicate new loss of an edge:
	max_edges--;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Add_edge : add new edge to end of edge array.
//	if the total edge array is too small to hold the new edge,
//      the edge array is enlarged to accomodate new edge.
//	ASSUMES: memory has already been allocated for new edge elt.

void    graph::Add_edge(edge *new_edge){
	if(numedges >= max_edges){
	   // if not enough room for new edge, increase array size:
		enlarge_edge_array_by(1);
	  };
	edges[numedges]=new_edge;
	numedges++;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// enlarges vertex array by given increment (parameter).

int graph::enlarge_vertex_array_by(int enlargement){
	max_edges += enlargement;
	vector **tmp = (vector**) new vector[max_edges];
	if(tmp==NULL){
		return NULL;
	  };
      // copy old vertex list.
	for(int i=0;i<numedges;i++){
	    tmp[i]= new vector(*pts[i]);
	 };
      // delete old data:
	for(i=0;i<numedges;i++){
	    delete pts[i];
	 };
	delete (vector**) pts;
	// reassign old pointer
	pts=tmp;
	return 1;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// enlarges point array to new size.

int graph::enlarge_vertex_array_to(int new_size){
	assert(new_size > max_edges);
	max_edges=new_size;
	vector **tmp = (vector**) new vector[max_edges];

      // copy old vertex list.
	for(int i=0;i<numedges;i++){
		tmp[i]= new vector(*pts[i]);
	   };
      // delete old data:
	for(i=0;i<numedges;i++){
		delete pts[i];
	   };
	delete (vector**) pts;
	pts = tmp;
	return 1;
};


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// increases size of memory allocated to edge array by 'enlargement'

int graph::enlarge_edge_array_by(int enlargement){
     max_edges+=enlargement;
     edge** tmp = (edge**) new edge[max_edges+enlargement];
     if(tmp == NULL){
	return 0;
       }
    // copy all old data:
     for(int i=0;i<numedges;i++){
	tmp[i]=new edge;
	tmp[i]->from = edges[i]->from;
	tmp[i]->to = edges[i]->to;
       };
    // delete all old data:
     for(i=0;i<numedges;i++){
	delete edges[i];
       };
	delete (edge**) edges;
     // set (internal pointer array to edges) to newly allocated array.
       edges=tmp;
       return 1;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// allocates more memory to increase the size of the edge pointer array.

int graph::enlarge_edge_array_to(int new_size){
      assert(new_size > max_edges);
      max_edges=new_size;
    // allocate space for new edge array of pointers to edges.
	edges = (edge**) new edge[new_size];
	assert(edges != NULL);
    // copy all old data into new array;
   /*	for(int i=0;i<numedges;i++){
		tmp[i] = new edge;
		tmp[i]->from = edges[i]->from;
		tmp[i]->to = edges[i]->to;
	   };
      // delete old edge array of pointers (and the edges themselves).
	for(i=0;i<numedges;i++){
		delete edges[i];
	   };*/
   //	delete (edge**) edges;
      // set edges to new larger array:
      //	edges=tmp;
      return 1;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// graph constructor ( copied points from given polygon)

graph::graph(const polygon& copy){
	int i=0;
	point *trav;
	numpts=copy.Numpts();
	max_pts=numpts;
   // allocate memory for point array:
	pts = (vector**) new vector[max_pts];
   // since all polygon lists are drawn first to second edge ..
   // second to last edge to last edge and last to first edge.
   // the number of edges is = the number of points.
	numedges=numpts;
	max_edges=numpts;
      edges = (edge**) new edge[max_edges];
    // copy all polygon data:
	trav = copy.Head();
	while(trav != NULL){
	    pts[i]= new vector;
	    *pts[i] = trav->coords();
	    i++;
	  };
    // set all edge indices:
	for(int i2=0;i2<numedges;i2++){
	   edges[i2]= new edge;
	   edges[i2]->from=i2;
	   edges[i2]->to=i2+1;
	 }
	edges[numedges]->to=0;
     initialized=TRUE;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// constructor, allocates memory for arrays to hold correct number of
// points and edges.  Allows for allocation of more memory than is neccesary
// for easier growth of graph object.

graph::graph(int num_pts,int num_edges,int maxpts,int maxedges){
   numpts=num_pts;
   numedges=num_edges;
   max_pts=maxpts;
   max_edges=maxedges;
   pts = (vector**) new vector[max_pts];
   edges = (edge**) new edge[max_edges];
   initialized=FALSE;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// copy constructor
//

graph::graph(graph& copy){
	numpts=copy.Numpts();
	numedges=copy.Numedges();
	max_pts=copy.Max_pts();
	max_edges=copy.Max_edges();

	Tx=copy.Tx;
	Ty=copy.Ty;
	colored=copy.colored;
    // allocate space for arrays.
	pts = (vector**) new vector[max_pts];
	edges = (edge**) new edge[max_edges];

    // create copies of individual vector,edge arrays.
	for(int i=0;i<numpts;i++){
		 pts[i]=new vector(copy[i]);
     //		*pts[i]=copy.pt_array(i);
	  };
	for(int i2=0;i2<numedges;i2++){
		 edges[i2]=new edge;
		 edges[i2]->to=copy.edge_array(i2).to;
		 edges[i2]->from=copy.edge_array(i2).from;
		 edges[i2]->color=copy.edge_array(i2).color;
	  };
	initialized=TRUE;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// constructor, allocates memory for arrays to hold correct number of
// points and edges.

graph::graph(int num_pts,int num_edges){
   numpts=0;
   numedges=0;
   max_pts=num_pts;
   max_edges=num_edges;
   pts = (vector**) new vector[max_pts];
   edges = (edge**) new edge[max_edges];
   initialized=FALSE;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// default constructor: init.s NULL object (no edges, no vertices)

graph::graph(){
    numpts=0;
    numedges=0;
    max_edges=0;
    max_pts=0;
    initialized = FALSE;
};