// MATRIX.H version 1.b
// Gregory Ecker
// 7/13/94
// Graphics programming.
// Professor Velho
// Header file for matrix + vector classes.
//
// Each class contains member functions for scaling,translation,rotation
// and dot products for matrices (4x4) and vectors (1x4)

#ifndef _MATRIX_H
#define _MATRIX_H

#include "commdefs.h"
#include <math.h> // cos(),sin()

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

class vector {
 friend matrix;
 public:
// constructors
	vector();
	vector(float,float,float,float);
	vector(const vector&);          // copy constructor

// Dot products:
vector  operator*(matrix&)const;	// vector-matrix multiplication.
float   operator*(vector&)const;	// vector-vector multiplication.

// Standard class operators:
 inline vector& operator=(const vector&); // vector assignment op.

 inline float& operator[](const int);   // returns scalar component of vector
 inline float operator[](const int) const; // const version (Rval only)
 inline	int operator==(const vector&) const; // vector comparison ( normalized ).
 inline	int operator!=(const vector&) const;
 private:
	float coords[4];
};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

class matrix {
 friend vector;
 public:
// constructors:
	matrix();
	matrix(const float array[4][4]);
	matrix(matrix&);

matrix& Identity(void);
// Translation member functions:
// returns transl. matrix, and changes *this object to translation mat.
 inline matrix& translate(float dx,float dy,float dz);
 inline matrix& translate(vector v);

// Scaling member functions:
// returns scaling matrix, and changes *this object to scaling mat.
 inline matrix& scale(float sx,float sy,float sz);
 inline matrix& scale(vector v);

// Rotation member functions:
// returns rotation matrix, and changes *this object to rotation mat.
 inline matrix& Xrotate(float theta); // returns matrix for x-rotation.
 inline matrix& NXrotate(float theta); // returns matrix for neg. x-rotation.
 inline matrix& Yrotate(float theta); // returns matrix for y-rotation.
 inline matrix& NYrotate(float theta); // returns matrix for neg. y-rotation.
 inline matrix& Zrotate(float theta); // returns matrix for z-rotation.
 inline matrix& NZrotate(float theta); // returns matrix for neg. z-rotation.

// mirroring member functions:
inline matrix& Ymirror(void);

//  perspective Projection member function:
//  returns matrix for perspective projection dependent upon d
//	(the distance from the center of projection).
 inline matrix& Project(float d);

// dot product:
 matrix operator*(matrix&);	       // matrix X matrix multiply

// standard class operators:
 inline void operator=(const matrix&); // assignment op.
 int matrix::operator==(const matrix& m) const; // equality op.
 int matrix::operator!=(const matrix& m) const; // inequality op.
 inline float& operator()(int,int); // returns lval scalar component of matrix
 inline float  operator()(int,int) const; // returns rval
// internal representation of matrix:
 private:
  float mat4x4[4][4];          // 4x4 array of floats.
};

#endif