// Gregory Ecker 7/18/94
// POINT CLASS  version 1.a
// point primitive class
// polygon class contains a list of point class objects.
#ifndef _POINT_H
#define _POINT_H
#include "commdefs.h"
#include "matrix.h" // for vector, matrix class def.s
#include <mem.h>    // for null pointer, memory handling

// define NDEBUG to get rid of all assertion tests (boundschecking)
// #define NDEBUG
#include <assert.h> // bounds checking assertions.
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// class 'point' is a shell for the vector class, to be used by
// higher order shape constructs (i.e. polygon)

class point {
  public:
inline point();                        // default constructor.
inline point(const point&);            // copy constructor.
inline point(const vector&);           // construct a point from a vector.
inline point(float,float,float,float); // construct a point from floats.

// construct a point from floats. and initialize next field to a point.
inline point(float,float,float,float,point*);

inline point* Next(void){ return next;}; // returns address of next point.
inline void SetNext(point* p){        	// assign new next field.
		 next=p; };

inline void Apply(matrix&);   //# applies given matrix to internal vector.

//inline vector operator*(const matrix) const; // returns dot product (V x M)

inline point& operator=(const point&);  //# copies a point,keeps same next field.
inline point& operator=(const vector&); //# copies vector components.

inline float  operator[](int) const; // rval [] op.
inline float& operator[](int);       // lval (non-const),assignment

inline int  Id(void)const{ return (id); }; // returns draw id for point.
inline void SetId(int i){ id=i; };	   // sets draw id for point.

// coordinate components:
inline vector coords(void) const;          // returns copy of internal vec.
inline float  x() const { return rep[0]; };
inline float  y() const { return rep[1]; };
inline float  z() const { return rep[2]; };
inline float  W() const { return rep[3]; };

// public data
   float theta;
   int axis;
   int color;    // for use when point is not grouped (alone);

 private:
	int id;	      // id for point (for use with polygon drawing).
	vector rep;
	point *next;  // for use as a list of points.
};

#endif
